import { ModalFormData, MessageFormData, ActionFormData } from "@minecraft/server-ui"
import { apiWaypointEntity } from "../lib/apiwaypoint/waypointEntity.js"
import { apiWaypointInfo } from "../lib/apiwaypoint/waypointInfo.js"
import { apiFormat } from "../lib/apiFormat.js"
import { waypointOptions } from "./configUi.js"
import { apiInfo } from "../lib/apiInfo.js"
import { world } from "@minecraft/server"

// Main Menu
export function waypointMenu(player){
    new ActionFormData()
    .title("waypoint.ui.config.title")
    .button("waypoint.ui.config.button.create")
    .button("waypoint.ui.config.button.edit")
    .button("waypoint.ui.config.button.remove")
    .button("waypoint.ui.config.button.options")
    .show(player).then(r => {
        if(r.canceled) return
        waypointUi[r.selection](player, apiInfo.getAllWaypoints(player))
    })
}

const icons = [ "waypoint.ui.create.icon.0", "waypoint.ui.create.icon.1", "waypoint.ui.create.icon.2", "waypoint.ui.create.icon.3", "waypoint.ui.create.icon.4", "waypoint.ui.create.icon.5", "waypoint.ui.create.icon.6", "waypoint.ui.create.icon.7" ]

export const waypointUi = new class waypointUi {
    // Create Waypoint
    0(player, waypoints, edit){
        const identifier = !edit ? "create" : "edit"
        new ModalFormData()
        .title(`waypoint.ui.${identifier}.title`)
        .textField("waypoint.ui.create.textfieldText", "waypoint.ui.create.textfieldHolder", !edit ? undefined : edit.id)
        .dropdown("waypoint.ui.create.dropdownText", icons, !edit ? 0 : edit.icon)
        .slider({translate: "waypoint.ui.create.sliderR"}, 0, 255, 1, !edit ? Math.floor(Math.random() * 255) : edit.rgb.r)
        .slider({translate: "waypoint.ui.create.sliderG"}, 0, 255, 1, !edit ? Math.floor(Math.random() * 255) : edit.rgb.g)
        .slider({translate: "waypoint.ui.create.sliderB"}, 0, 255, 1, !edit ? Math.floor(Math.random() * 255) : edit.rgb.b)
        .toggle("waypoint.ui.create.toggle", !edit ? true : edit.visible)
        .submitButton("waypoint.ui.create.buttonSave")
        .show(player).then(r => {
            if(r.canceled) return edit ? this[1](player, waypoints) : player.onScreenDisplay.setActionBar({translate: `waypoint.warning.cancelCreatePoint`})
            if(!r.formValues[0]) return player.onScreenDisplay.setActionBar({translate: `waypoint.warning.failCreatePoint`})
            if(edit) return apiWaypointInfo.editWaypoint(player, waypoints, edit.id, r.formValues, edit.index)
            apiWaypointInfo.createWaypoint(player, r.formValues, waypoints)
        })
    }

    // Edit Waypoints
    1(player, waypoints){
        if(waypoints.length < 1) return player.onScreenDisplay.setActionBar({translate: "waypoint.warning.dontHavePoints"})
        const buttons = waypoints.map(way => ({id: way.id, icon: way.icon, pos: apiFormat.formatPos(way.pos)}))
        const form = new ActionFormData()
        .title("waypoint.ui.edit.title")
        buttons.forEach(button => { form.button(`${button.id}\n${Math.floor(button.pos.x)}, ${button.pos.y}, ${Math.floor(button.pos.z)}`, `textures/entity/icon_${button.icon}`) })
        form.show(player).then(r => {
            if(r.canceled) return
            const point = waypoints[r.selection]
            this[0](player, waypoints, {id: point.id, icon: point.icon, rgb: {r: point.rgb.r, g: point.rgb.g, b: point.rgb.b}, visible: point.visible, index: r.selection})
        })
    }

    // Remove Waypoint
    2(player, waypoints){
        if(waypoints.length < 1) return player.onScreenDisplay.setActionBar({translate: "waypoint.warning.dontHavePoints"})
        new ModalFormData()
        .title("waypoint.ui.delete.title")
        .dropdown("waypoint.ui.delete.dropdown", waypoints.map(way => way.id))
        .submitButton("waypoint.ui.delete.buttonDelete")
        .show(player).then(r => {
            if(r.canceled) return player.onScreenDisplay.setActionBar({translate: "waypoint.warning.dontDelete"})
            const point = waypoints[r.formValues[0]]
            new MessageFormData()
            .title("waypoint.ui.delete.title")
            .body({translate: "waypoint.ui.delete.confirm", with: [point.id, `${point.pos.x -0.5}, ${point.pos.y}, ${point.pos.z -0.5}`]})
            .button1("waypoint.ui.yes")
            .button2("waypoint.ui.no")
            .show(player).then(r2 => {
                if(r2.canceled) return player.onScreenDisplay.setActionBar({translate: "waypoint.warning.dontDelete"})
                if(r2.selection == 1) return player.onScreenDisplay.setActionBar({translate: "waypoint.warning.dontDelete"})
                apiWaypointEntity.removeWaypoint(player, waypoints[r.formValues[0]].id)
                waypoints.splice(r.formValues[0], 1)
                apiInfo.saveAllPoints(player, waypoints)
                player.onScreenDisplay.setActionBar({translate: "waypoint.warning.deleted", with: [point.id]})
            })
        })
    }

    // Waypoint Options
    3(player){
        new ActionFormData()
        .title("waypoint.ui.options.title")
        .button("waypoint.ui.options.general.title")
        .button("waypoint.ui.options.recover.title")
        .show(player).then(r => {
            if(r.canceled) return
            waypointOptions[r.selection](player)
        })
    }
}